//
//  MyProfileTableViewController.swift
//  Tusker
//
//  Created by Shadowfacts on 11/24/18.
//  Copyright © 2018 Shadowfacts. All rights reserved.
//

import UIKit
import SwiftUI

class MyProfileTableViewController: ProfileTableViewController {

    init(mastodonController: MastodonController) {
        super.init(accountID: nil, mastodonController: mastodonController)
        
        title = "My Profile"
        tabBarItem.image = UIImage(systemName: "person.fill")
        
        
        mastodonController.getOwnAccount { (account) in
            self.accountID = account.id
            
            _ = ImageCache.avatars.get(account.avatar, completion: { [weak self] (data) in
                guard let self = self, let data = data, let image = UIImage(data: data) else { return }
                DispatchQueue.main.async {
                    let size = CGSize(width: 30, height: 30)
                    let tabBarImage = UIGraphicsImageRenderer(size: size).image { (_) in
                        image.draw(in: CGRect(origin: .zero, size: size))
                    }
                    let alwaysOriginalImage = tabBarImage.withRenderingMode(.alwaysOriginal)
                    self.tabBarItem.image = alwaysOriginalImage
                }
            })
        }
        
        navigationItem.rightBarButtonItem = UIBarButtonItem(title: "Preferences", style: .plain, target: self, action: #selector(preferencesPressed))
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
    }
    
    @objc func preferencesPressed() {
        present(PreferencesNavigationController(mastodonController: mastodonController), animated: true)
    }
    
    @objc func closePreferences() {
        dismiss(animated: true)
    }

}
